''
''
'' Remote Controlled Room Lamp
'' -- Jason Dorie --               
''               
''

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  TableCount = 1024                                     'Number of entries in the power to brightness table
  Stagger = 128                                         'Stagger between LED sections when fading in / out
  Channels = 16                                         'Total number of channels of LEDs
  MaxBright = TableCount + Channels*Stagger             'Maximum overall brightness value = 16 channels * Stagger


VAR
  long FadeMode
  long FaderStack[24]
  long Brightness
  long BrightTarget
  long FadeDelay
  word BrightTable[TableCount]


OBJ
  IR:   "jm_sircs_rx.spin"
  PWM:  "TLC5940-Driver-asm.spin"
  Flt:  "Float32.spin"
  'Dbg:  "FullDuplexSerial.spin"


PUB Main | value, loop, loop2, chan, cog, TimeVal, TestVal, code, bc, device

  'Dbg.Start( 31, 30, 0, 115200 )
  Flt.Start                                             'Init the float cog
  InitBrightTable                                       'Initialize the brightness power table using the float cog
  Flt.Stop                                              'Kill the float cog as it's not used again
  
  
  IR.Init( 24 )                                         'Run the IR receiver on pin 24                        

  repeat loop from 0 to constant(Channels-1)            'Init the brightness table to zero
    PWM.SetChValue( loop , 0 )
  
  cog := PWM.StartTLC5940(11, 13, 10, 9, 8, 0, 12)

  FadeDelay := 100_000                                  'Speed to run the fade loop
  Brightness := 0                                       'Initial brightness value on startup                        
  BrightTarget := 0                                     'Initial brightness target value on startup (will fade to this)
  cognew( RunFader, @FaderStack )                       'Start the cog that runs the brightness fader

  
  repeat
    code := ir.getir                                    'Read the remote
    bc := ir.bitcount                                   'cache the bitcount
    device := code >> 7                                 'cache the device ID from the IR result

    if( bc == 12 AND device == 1 )                      'if the device is a TV and it's a 12bit code...
      code &= 127                                       '...mask off the code value and deal with it

      'There are a total of 9 digits on the remote that will set a brightness value, and the zero.
      'Therefore, each digit is set to N x 1/9th maximum brightness, where N is the digit.  This
      'gives close to linear response over the range of keys pressed and acts as a coarse setting.
      'Volume up / down controls brightness in fine (single unit) increments.

      case code
        9: 'digit 0
          BrightTarget := constant(MaxBright*0/9)

        0: 'digit 1
          BrightTarget := constant(MaxBright*1/9)

        1: 'digit 2
          BrightTarget := constant(MaxBright*2/9)

        2: 'digit 3
          BrightTarget := constant(MaxBright*3/9)

        3: 'digit 4
          BrightTarget := constant(MaxBright*4/9)

        4: 'digit 5
          BrightTarget := constant(MaxBright*5/9)
          
        5: 'digit 6
          BrightTarget := constant(MaxBright*6/9)
          
        6: 'digit 7
          BrightTarget := constant(MaxBright*7/9)
          
        7: 'digit 8
          BrightTarget := constant(MaxBright*8/9)
          
        8: 'digit 9
          BrightTarget := constant(MaxBright*9/9)

        16: 'Ch+                                        'Change the fade mode                        
          FadeMode := (FadeMode+1) & 3

        17: 'Ch-                                        'Change the fade mode                        
          FadeMode := (FadeMode-1) & 3
          
        18: 'Vol+
          if( BrightTarget < MaxBright )
            BrightTarget += 32

        19: 'Vol-    
          if( BrightTarget > 0 )
            BrightTarget -= 32

    waitcnt( constant(80_000_000 / 80) + cnt )          'Run the update loop at a max of 80 hz                        


           
PUB InitBrightTable | index, val, gamma

''This function generates the table of brightness to power values.  Increasing
''the brightness values linearly results in a logarithmic increase in brightness.
''We want the brightness of the lamp to appear linear, so it's an exponential
''table.  This COULD be stored as a "baked" table of values, but this approach
''is more flexible. 


  BrightTable[0] := 0
  repeat index from 1 to constant(TableCount-1)
    val := Flt.FMul( constant(1.0/float(TableCount)) , Flt.FFloat(index) )      'Bright value ranges from 0.0 to 1.0
    gamma := Flt.Pow(val , 2.0 )                                                'Convert from linear to "perceived linear" 
    BrightTable[index] := Flt.FTrunc( Flt.FMul( gamma , 4095.0 ) ) <# 4095      'PWM values are from 0 to 4095



PUB RunFader
  repeat
    Update
    waitcnt( FadeDelay + cnt )


PUB Update | index, val

''This function performs the various fades (top down, bottom up, middle out, or ends in)
''and deals with staggering the different LED channels.  A stagger value of 0 would fade
''all channels in at the same time.

  case FadeMode
    0:
      repeat index from 0 to 15
        val := ((Brightness - (index*Stagger)) #> 0) <# constant(TableCount-1)
        PWM.SetChValue( index, BrightTable[val] )

    1:
      repeat index from 0 to 7
        val := ((Brightness - (index*Stagger*2)) #> 0) <# constant(TableCount-1)
        PWM.SetChValue( 7-index, BrightTable[val] )
        PWM.SetChValue( 8+index, BrightTable[val] )

    2:
      repeat index from 0 to 15
        val := ((Brightness - (index*Stagger)) #> 0) <# constant(TableCount-1)
        PWM.SetChValue( 15-index, BrightTable[val] )
        
    3:
      repeat index from 0 to 7
        val := ((Brightness - (index*Stagger*2)) #> 0) <# constant(TableCount-1)
        PWM.SetChValue( index, BrightTable[val] )
        PWM.SetChValue( 15-index, BrightTable[val] )

  ''Move the brightness value toward the brightness target value       
  if( Brightness < BrightTarget )
    Brightness++
  if( Brightness > BrightTarget )
    Brightness--

  PWM.UpdateScreen                                      'Tell the TLC5940 cog to commit the values to the chip    




   
PUB PWMPin( Pin, Period ) | Duty

''This function PWMs a single pin used as a "heartbeat" to make sure everything
''is working.  It's connected to a single LED on the main board as a simple
''status indicator.  Currently unused.

  DIRA[Pin] := 1

  repeat
    repeat Duty from 0 to Period

      OUTA[Pin] := 1
      waitcnt( 400 + Duty * 10 + cnt )

      OUTA[Pin] := 0
      waitcnt( 400 + (Period-Duty) * 10 + cnt)

    repeat Duty from Period to 0

      OUTA[Pin] := 1
      waitcnt( 400 + Duty * 10 + cnt )

      OUTA[Pin] := 0
      waitcnt( 400 + (Period - Duty) * 10 + cnt)